<?php
/*******************************************************************************
	This is script for installing Related Topics

	ATTENTION: If you are trying to INSTALL this package, please access
	it directly, with a URL like the following:
		http://www.yourdomain.tld/forum/related_topics.php (or similar.)

*******************************************************************************/
global $db_prefix, $db_type;

// If we have found SSI.php and we are outside of ELK, then we are running standalone.
if (file_exists(dirname(__FILE__) . '/SSI.php') && !defined('ELK'))
	require_once(dirname(__FILE__) . '/SSI.php');

// If we are outside ELK and can't find SSI.php, then throw an error
elseif (!defined('ELK'))
	die('<b>Error:</b> Cannot install - please verify you put this file in the same place as Elkarte\'s SSI.php.');

if (ELK == 'SSI')
	db_extend('packages');

$index_type = $db_type == 'mysql' ? 'fulltext' : '';

$relatedSettings = array(
	'relatedTopicsEnabled' => array(0, false),
	'relatedTopicsCount' => array(5, false),
	'relatedIndex' => array($index_type, false),
	'relatedIgnoredboards' => array('', false),
);

$tables = array(
	'related_topics' => array(
		'name' => 'related_topics',
		'columns' => array(
			array(
				'name' => 'id_topic_first',
				'type' => 'int',
				'null' => false,
				'unsigned' => true,
			),
			array(
				'name' => 'id_topic_second',
				'type' => 'int',
				'null' => false,
				'unsigned' => true,
			),
			array(
				'name' => 'score',
				'type' => 'float',
				'null' => false,
				'unsigned' => true,
			),
		),
		'indexes' => array(
			array(
				'type' => 'primary',
				'columns' => array('id_topic_first', 'id_topic_second')
			),
		)
	),
);

doSettings($relatedSettings);
doTables($tables);

// Functions
function doTables($tables, $columnRename = array())
{
	global $db_prefix, $db_type, $db_show_debug;
	$db = database();
	$dbtbl = db_table();

	$log = array();
	$existingTables = $db->db_list_tables();

	foreach ($tables as $table)
	{
		$table_name = $table['name'];

		$tableExists = in_array($db_prefix . $table_name, $existingTables);

		// Create table
		if (!$tableExists && empty($table['elk']))
			$dbtbl->db_create_table('{db_prefix}' . $table_name, $table['columns'], $table['indexes']);

		// Update table
		else
		{
			$currentTable = $dbtbl->db_table_structure('{db_prefix}' . $table_name);

			// Renames in this table?
			if (!empty($table['rename']))
			{
				foreach ($currentTable['columns'] as $column)
				{
					if (isset($table['rename'][$column['name']]))
					{
						$old_name = $column['name'];
						$column['name'] = $table['rename'][$column['name']];

						$dbtbl->db_change_column('{db_prefix}' . $table_name, $old_name, $column);
					}
				}
			}

			// Global renames? (should be avoided)
			if (!empty($columnRename))
			{
				foreach ($currentTable['columns'] as $column)
				{
					if (isset($columnRename[$column['name']]))
					{
						$old_name = $column['name'];
						$column['name'] = $columnRename[$column['name']];
						$dbtbl->db_change_column('{db_prefix}' . $table_name, $old_name, $column);
					}
				}
			}

			// Check that all columns are in
			foreach ($table['columns'] as $id => $col)
			{
				$exists = false;

				// TODO: Check that definition is correct
				foreach ($currentTable['columns'] as $col2)
				{
					if ($col['name'] === $col2['name'])
					{
						$exists = true;
						break;
					}
				}

				// Add missing columns
				if (!$exists)
					$dbtbl->db_add_column('{db_prefix}' . $table_name, $col);
			}

			// Remove any unnecassary columns
			foreach ($currentTable['columns'] as $col)
			{
				$exists = false;

				foreach ($table['columns'] as $col2)
				{
					if ($col['name'] === $col2['name'])
					{
						$exists = true;
						break;
					}
				}

				if (!$exists && isset($table['upgrade']['columns'][$col['name']]))
				{
					if ($table['upgrade']['columns'][$col['name']] == 'drop')
						$dbtbl->db_remove_column('{db_prefix}' . $table_name, $col['name']);
				}
				elseif (!$exists && !empty($db_show_debug) && empty($table['elk']))
					$log[] = sprintf('Table %s has non-required column %s', $table_name, $col['name']);
			}

			// Check that all indexes are in and correct
			foreach ($table['indexes'] as $id => $index)
			{
				$exists = false;

				foreach ($currentTable['indexes'] as $index2)
				{
					// Primary is special case
					if ($index['type'] == 'primary' && $index2['type'] == 'primary')
					{
						$exists = true;

						if ($index['columns'] !== $index2['columns'])
						{
							$dbtbl->db_remove_index('{db_prefix}' . $table_name, 'primary');
							$dbtbl->db_add_index('{db_prefix}' . $table_name, $index_info);
						}

						break;
					}
					// Make sure index is correct
					elseif (isset($index['name']) && isset($index2['name']) && $index['name'] == $index2['name'])
					{
						$exists = true;

						// Need to be changed?
						if ($index['type'] != $index2['type'] || $index['columns'] !== $index2['columns'])
						{
							$dbtbl->db_remove_index('{db_prefix}' . $table_name, $index['name']);
							$dbtbl->db_add_index('{db_prefix}' . $table_name, $index_info);
						}

						break;
					}
				}

				if (!$exists)
					$dbtbl->db_add_index('{db_prefix}' . $table_name, $index_info);
			}

			// Remove unnecassary indexes
			foreach ($currentTable['indexes'] as $index)
			{
				$exists = false;

				foreach ($table['indexes'] as $index2)
				{
					// Primary is special case
					if ($index['type'] == 'primary' && $index2['type'] == 'primary')
						$exists = true;
					// Make sure index is correct
					elseif (isset($index['name']) && isset($index2['name']) && $index['name'] == $index2['name'])
						$exists = true;
				}

				if (!$exists)
				{
					if (isset($table['upgrade']['indexes']))
					{
						foreach ($table['upgrade']['indexes'] as $index2)
						{
							if ($index['type'] == 'primary' && $index2['type'] == 'primary' && $index['columns'] === $index2['columns'])
								$dbtbl->db_remove_index('{db_prefix}' . $table_name, 'primary');
							elseif (isset($index['name']) && isset($index2['name']) && $index['name'] == $index2['name'] && $index['type'] == $index2['type'] && $index['columns'] === $index2['columns'])
								$dbtbl->db_remove_index('{db_prefix}' . $table_name, $index['name']);
							elseif (!empty($db_show_debug))
								$log[] = $table_name . ' has Unneeded index ' . var_dump($index);
						}
					}
					elseif (!empty($db_show_debug))
						$log[] = $table_name . ' has Unneeded index ' . var_dump($index);
				}
			}
		}
	}

	if (!empty($log))
		log_error(implode('<br />', $log));

	return $log;
}

function doSettings($addSettings)
{
	global $modSettings;

	$update = array();

	foreach ($addSettings as $variable => $value)
	{
		list ($value, $overwrite) = $value;

		if ($overwrite || !isset($modSettings[$variable]))
			$update[$variable] = $value;
	}

	if (!empty($update))
		updateSettings($update);
}

function doPermission($permissions)
{
	$db = database();

	$perm = array();

	foreach ($permissions as $permission => $default)
	{
		$result = $db->query('', '
			SELECT COUNT(*)
			FROM {db_prefix}permissions
			WHERE permission = {string:permission}',
			array(
				'permission' => $permission
			)
		);

		list ($num) = $db->fetch_row($result);

		if ($num == 0)
		{
			foreach ($default as $grp)
				$perm[] = array($grp, $permission);
		}
	}

	if (empty($perm))
		return;

	$db->insert('insert',
		'{db_prefix}permissions',
		array(
			'id_group' => 'int',
			'permission' => 'string'
		),
		$perm,
		array()
	);
}

function updateAdminFeatures($item, $enabled = false)
{
	global $modSettings;

	$admin_features = isset($modSettings['admin_features']) ? explode(',', $modSettings['admin_features']) : array('cd,cp,k,w,rg,ml,pm');

	if (!is_array($item))
		$item = array($item);

	if ($enabled)
		$admin_features = array_merge($admin_features, $item);
	else
		$admin_features = array_diff($admin_features, $item);

	updateSettings(array('admin_features' => implode(',', $admin_features)));

	return true;
}

// Define the hooks
$hook_functions = array(
	'integrate_admin_areas' => 'RelatedTopics_Admin_Menu|SOURCEDIR/addons/RelatedTopics.php',
	'integrate_modify_modifications' => 'RelatedTopicsAdminSettings|SOURCEDIR/addons/RelatedTopics.php',
);

// Adding or removing them?
if (!empty($context['uninstalling']))
	$call = 'remove_integration_function';
else
	$call = 'add_integration_function';

// Do the deed
foreach ($hook_functions as $hook => $function)
	$call($hook, $function);

if (ELK == 'SSI')
   echo 'Congratulations! You have successfully installed this mod!';

?>